using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;

public class GameScript : MonoBehaviour {

	public Text textToType;
	public Image timeBar;
	public RectTransform canvas;
	public Text scoreText, highScoreText;
	public GameObject losePopUp;
	public Text losePopUpText;
	public GameObject mantra;
	public Animator servant1Anim, servant2Anim, servant3Anim, servant4Anim;
	public GameObject weighObject;
	public SpriteRenderer weighSprite;
	public BoxCollider2D weighCollider;
	public Rigidbody2D weighRigidbody;
	public Sprite stone, chicken;
	public Transform ghostTransform;
	public GhostScript ghostScript;
	public AudioSource audioSource;
	public AudioClip []sfx;
	// 0. Type
	// 1. Lose
	// 2. Highscore
	// 3. Wrong
	// 4. Boom

	bool gameOver;
	int success;
	int level;
	int randomText;
	int currentTextIndex;
	string[][] listText;
	bool delayRunning;
	bool startDelay;
	public float maxTime;
	public float time;
	int score;
	bool newHighscore;
	List<int> appearedList;
	GameObject mantraLeft, mantraRight;

	// Use this for initialization
	void Start () {
		listText = new string[5][];
		for(int i=0; i<listText.Length; i++)
			listText[i] = new string[20];

		TextAsset textAsset = (TextAsset)Resources.Load ("ListText");
		string [] list = textAsset.text.Split ("\n"[0]);

		int k = 0;
		for (int i=0; i<listText.Length; i++) {
			for(int j=0; j<listText[i].Length; j++){
				listText[i][j] = list[k++];
			}
		}

		startDelay = true;
		level = 0;
		success = 0;
		score = 0;
		highScoreText.text = "High Score: " + DataScript.playerData.Highscore;
		maxTime = 3f;
		time = maxTime;
		timeBar.rectTransform.sizeDelta = new Vector2(time / maxTime * canvas.sizeDelta.x, timeBar.rectTransform.sizeDelta.y);
		timeBar.color = Color.Lerp(Color.red, Color.green, time/maxTime);
		appearedList = new List<int>();
		randomNewText (level);
	}
	
	// Update is called once per frame
	void Update () {
		if (gameOver)
			return;

		if (Input.anyKeyDown && !delayRunning) {
			startDelay = false;
			if(Input.inputString == listText [level][randomText][currentTextIndex].ToString()){
				currentTextIndex++;
				textToType.text = "<color=#BCFF00FF>" + listText[level][randomText].Substring(0,currentTextIndex) + "</color>" + listText[level][randomText].Substring(currentTextIndex);
				audioSource.clip = sfx[0];
				audioSource.Play();

				if(currentTextIndex == listText[level][randomText].Length - 1){
					audioSource.clip = sfx[4];
					audioSource.Play();
					success++;
					score++;
					scoreText.text = "Score: " + score;
					currentTextIndex = 0;
					weighObject.SetActive(true);
					showMantra();

					if(success >= 10){
						level++;
						success = 0;
						appearedList.Clear();
					}

					StartCoroutine(newTextShowDelay());
				}
			}
			else{
				if(!Input.GetKeyDown(KeyCode.LeftShift) && !Input.GetKeyDown(KeyCode.RightShift) && 
				   !Input.GetKeyDown(KeyCode.CapsLock) && 
				   !Input.GetKeyDown(KeyCode.LeftControl) && !Input.GetKeyDown(KeyCode.RightControl)){
					audioSource.clip = sfx[5];
					audioSource.Play();

					weighCollider.isTrigger = true;
					weighSprite.sprite = chicken;
					weighObject.SetActive(true);
					ghostScript.moveUpFast = true;
					ghostScript.lerpValue = 0;
					ghostScript.currentPosition = ghostTransform.position;

					currentTextIndex++;
					gameOver = true;
					StartCoroutine(textBlink());
				}
			}
		}

		if (!startDelay && !delayRunning && !gameOver) {
			time -= Time.deltaTime;
			timeBar.rectTransform.sizeDelta = new Vector2 (time / maxTime * canvas.sizeDelta.x, timeBar.rectTransform.sizeDelta.y);
			timeBar.color = Color.Lerp (Color.red, Color.green, time / maxTime);
		}

		if (time <= 0)
			gameOver = true;

		if (gameOver) {
			StartCoroutine(showLosePopUp());
			servant1Anim.SetBool("GameOver", true);
			servant2Anim.SetBool("GameOver", true);
			servant3Anim.SetBool("GameOver", true);
			servant4Anim.SetBool("GameOver", true);

			if(score > DataScript.playerData.Highscore){
				losePopUpText.text = "Muahahahaha\n\nYou failed to keep\nthe ghost\nunsummoned!\n\nBut at least u got a new highscore~\n";
				newHighscore = true;
				DataScript.playerData.Highscore = score;
				DataScript.save();
			}else{
				audioSource.Play();
			}
		}
	}

	void randomNewText(int level){
		randomText = Random.Range (0, 20);
		while (appearedList.Contains(randomText)) {
			randomText = Random.Range (0, 20);
		}

		appearedList.Add (randomText);
		if(appearedList.Count >= 20)
			appearedList.Clear();

		textToType.text = listText [level] [randomText];
	}

	IEnumerator newTextShowDelay(){
		delayRunning = true;
		yield return new WaitForSeconds(0.5f);

		randomNewText(level);
		delayRunning = false;
		time = 3f;
	}

	IEnumerator textBlink(){
		while (true) {
			textToType.text = "<color=#BCFF00FF>" + listText[level][randomText].Substring(0,currentTextIndex-1) + "</color>" + "<color=#FF2727FF>" + listText[level][randomText].Substring(currentTextIndex-1,1) + "</color>" + listText[level][randomText].Substring(currentTextIndex);
			yield return new WaitForSeconds(0.5f);
			textToType.text = "<color=#BCFF00FF>" + listText[level][randomText].Substring(0,currentTextIndex-1) + "</color>" + "<color=#FF272700>" + listText[level][randomText].Substring(currentTextIndex-1,1) + "</color>" + listText[level][randomText].Substring(currentTextIndex);
			yield return new WaitForSeconds(0.5f);
		}
	}

	IEnumerator showLosePopUp(){
		yield return new WaitForSeconds(1f);

		audioSource.clip = sfx[1];
		if (newHighscore) {
			audioSource.clip = sfx[2];
		} 
		audioSource.Play ();
		losePopUp.SetActive (true);
	}

	void showMantra(){
		mantraLeft = Instantiate(mantra) as GameObject;
		mantraRight = Instantiate(mantra) as GameObject;
		mantraLeft.transform.SetParent(canvas);
		mantraRight.transform.SetParent(canvas);
		mantraLeft.GetComponent<RectTransform>().anchoredPosition = new Vector2(-400f, -5f);
		mantraRight.GetComponent<RectTransform>().anchoredPosition = new Vector2(400f, -5f);
		mantraLeft.GetComponent<RectTransform>().localScale = new Vector2(1f, 1f);
		mantraRight.GetComponent<RectTransform>().localScale = new Vector2(1f, 1f);
		mantraRight.transform.SetParent(canvas);
		mantraLeft.GetComponent<Text>().text = listText[level][randomText];
		mantraRight.GetComponent<Text>().text = listText[level][randomText];
	}

	public void retryGame(){
		Application.LoadLevel (Application.loadedLevel);
	}

	public void quitGame(){
		Application.LoadLevel (1);
	}
}
